@@ -337,7 +337,7 @@ int distance2( pix *p1, struct box *box1,
 wchar_t ocr_db(struct box *box1) {
   int dd = 1000, dist = 1000;
   wchar_t c = UNKNOWN;
-  char buf[200];
+  unsigned char buf[200]; /* Oct08 JS: add unsigned to avoid UTF problems */
   Box *box2, *box3;
   
   if (!list_empty(&JOB->tmp.dblist)){ 
@@ -400,8 +400,8 @@ wchar_t ocr_db(struct box *box1) {
      * if not UTF-input-mode, we are in trouble?
      * ^a=0x01, ^b=0x02, ^e=05, ..., ToDo: meaning of no-input or <=space
      */
-    fgets(buf,200,stdin); /* including \n=0x0a */
-    dd=strlen(buf);
+    fgets((char *)buf,200,stdin); /* including \n=0x0a */
+    dd=strlen((char *)buf);
     /* output hexcode if verbose set */
     if (JOB->cfg.verbose) {
       fprintf(stderr, "\n# fgets [%d]:", dd);
@@ -410,7 +410,7 @@ wchar_t ocr_db(struct box *box1) {
       fprintf(stderr, "\n#");
     }
     /* we dont accept chars which could destroy database file */
-    for (i=0; i<dd; i++) if (buf[i]<32) break;
+    for (i=0; i<dd; i++) if (buf[i]<32) break; /* need unsigned char here */
     endchar=buf[i]; /* last char is 0x0a (ret) 0x00 (EOF) or 0x1b (alt+ret) */
     if (endchar==0x01) { i=0;JOB->cfg.mode&=~128; } /* skip all */
     buf[dd=i]=0; /* replace final 0x0a or other special codes */
@@ -429,7 +429,7 @@ wchar_t ocr_db(struct box *box1) {
       if (utf8_ok==1) { setac(box1, c, 100); } /* store single wchar */
       if (utf8_ok==0) {     /* store a string of chars (UTF8-string) */
         c='_'; /* what should we do with c? probably a bad idea? */
-        setas(box1, buf, 100);
+        setas(box1, (char *)buf, 100);
       }
       /* decide between
        *  0) just help gocr to find the results and (dont remember, 0x01)
