@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2007  Joerg Schulenburg
+Copyright (C) 2000-2008  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -46,7 +46,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 static void out_version(int v) {
   fprintf(stderr, " Optical Character Recognition --- gocr "
           version_string " " release_string "\n"
-          " Copyright (C) 2001-2007 Joerg Schulenburg\n"
+          " Copyright (C) 2001-2008 Joerg Schulenburg\n"
           " released under the GNU General Public License\n");
   /* as recommended, (c) and license should be part of the binary */
   /* no email because of SPAM, see README for contacting the author */
@@ -63,7 +63,7 @@ static void help(void) {
   fprintf(stderr,
 	  " using: gocr [options] pnm_file_name  # use - for stdin\n"
 	  " options (see gocr manual pages for more details):\n"
-	  " -h        - get this help\n"
+	  " -h, --help\n"
 	  " -i name   - input image file (pnm,pgm,pbm,ppm,pcx,...)\n"
 	  " -o name   - output file  (redirection of stdout)\n"
 	  " -e name   - logging file (redirection of stderr)\n"
