@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2007 Joerg Schulenburg
+Copyright (C) 2000-2008 Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -130,7 +130,7 @@ int get_least_line_indent(List * boxlist, int dx, int dy) {
 /* collect all the chars from the box tree and write them to a string buffer
    mo is the mode: mode&8 means, use chars even if unsure recognized
    ToDo: store full text(?), store decoded text+boxes+position chars (v0.4)
-         (HTML,UTF,ASCII,XML), not wchar incl. dexcriptions (at<95% in red)
+         (HTML,UTF,ASCII,XML), not wchar incl. descriptions (at<95% in red)
          remove decode(*c, job->cfg.out_format) from gocr.c!
          XML add alternate-tags, format tags and position tags 
    ToDo: better output XML to stdout instead of circumstantial store to lines
@@ -280,20 +280,19 @@ void store_boxtree_lines(int mo) {
         if (box2->num_ac>1) { /* ToDo: output a list of alternatives */
         }
       }
-      if (box2->c != UNKNOWN && box2->c!=0) {
+      if (box2->c != UNKNOWN  &&  box2->c != 0) {
         buffer=
           append_to_line(buffer,decode(box2->c,JOB->cfg.out_format),&len);
         if (box2->c >  ' ' &&
             box2->c <= 'z') i2++; /* count non-space chars */
-      } else { /* c == UNKNOWN */
+      } else { /* c == UNKNOWN or 0 */
         wchar_t cc; cc=box2->c;
         if (box2->num_ac>0 && box2->tas[0]
                 && (JOB->cfg.out_format!=XML || box2->tas[0][0]!='<')) {
-/*
+          /* output glued chars or ... (?) Jan08 */
           buffer=append_to_line(buffer,box2->tas[0],&len);
           j+=strlen(box2->tas[0]);
-*/
-        } else {
+        } else {  /* leave string empty? */
           /*
            buffer=
             append_to_line(buffer,decode(cc,JOB->cfg.out_format),&len);
