@@ -165,6 +165,10 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
       if(   loop(bp,dx/4,3*dy/10,dy,cs,1,DO)>0 ) // check start
       if(   loop(bp,dx-2,3*dy/4 ,dy,cs,1,UP)>0 ) // check end
       if( num_cross(dx/4,dx-2,3*dy/10,3*dy/4,bp,cs)==1 ) Break; // connected?
+      
+      // small fat s is very similar to rounded serif 5
+      //   but has a fat diagonal arrea (Oct08 JS)
+      if ( num_cross(dx/8,dx-1-dx/8,3*dy/10,3*dy/4,bp,cs)==1 ) ad=98*ad/100;
 
       if( dy>17 && dx>9 ) /* S */
       if(   loop(bp,   0,dy/2   ,dx,cs,0,RI)<dx/2
@@ -286,9 +290,13 @@ wchar_t ocr0n(ocr0_shared_t *sdata){
 //      j =loop(bp,0,2,dx,cs,0,RI); if( j>=x ) Break; x=j; // ~l
 //      j =loop(bp,0,   0,dx,cs,0,DO); if( !j  ) Break; // ~7
       if( !hchar ) // ~ right part of n
+      /* look for upper right side, not going from nw to se */ 
       if( loop(bp,dx-1,   1,dx,cs,0,LE)-dy/6
         > loop(bp,dx-1,dy/4,dx,cs,0,LE)
-       || get_bw(x1+1,x1+2,y0,y0+dy/8,box1->p,cs,1)==1 ) Break; // Mai00
+      // failes on small fonts with 1 point spaces between chars
+      //   also do not know what the purpose of this line was (JS Oct08)
+      // || get_bw(x1+1,x1+2,y0,y0+dy/8,box1->p,cs,1)==1
+        ) Break; // Mai00
       if( loop(bp,dx-1,3*dy/4,dx,cs,0,LE) > dx/2
        && get_bw(x1-dx/4,x1,y1-1,y1,box1->p,cs,1)==1 ) Break; // ~z Jun00
 
