@@ -1,6 +1,6 @@
 /*
 This is a Optical-Character-Recognition program
-Copyright (C) 2000-2007  Joerg Schulenburg
+Copyright (C) 2000-2008  Joerg Schulenburg
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
     v0.2.7 review by Bruno Barberi Gnecco
     v0.39  autoconf
     v0.41  fix integer and heap overflow, change color output
+    v0.46  fix blank spaces problem in filenames
  */
 
 #include <stdlib.h>
@@ -149,7 +150,7 @@ int readpgm(char *name, pix * p, int vvv) {
       }
       else {
         char *buf = (char *)malloc((strlen(pip)+strlen(name)+2));
-        sprintf(buf, "%s %s", pip, name);
+        sprintf(buf, "%s \"%s\"", pip, name); /* allow spaces in filename */
         if (vvv) {
           fprintf(stderr, "# popen( %s )\n", buf);
         }
@@ -284,7 +285,7 @@ int readpgm( char *name, pix *p, int vvv){
       if (!pip) {
         f1=fopen(name,"rb"); if (!f1) F1("opening file %s",name);
       } else {
-        sprintf(buf,"%s %s",pip,name); /* ToDo: how to prevent OVL ? */
+        sprintf(buf,"%s \"%s\"",pip,name); /* ToDo: how to prevent OVL ? */
         if (vvv) { fprintf(stderr,"# popen( %s )\n",buf); }
 #ifdef HAVE_POPEN
         f1=popen(buf,"r");
@@ -302,11 +303,11 @@ int readpgm( char *name, pix *p, int vvv){
   //    TXT   P1    P2    P3
   //    RAW   P4    P5    P6
   if (c1!='P' || c2 <'1' || c2 >'6') { 
-    fprintf(stderr,"\nread-PNM-error: file descriptor is %08x,"
-                   " position %ld", f1, ftell(f1));
+    fprintf(stderr,"\nread-PNM-error: file number is %2d,"
+                   " position %ld", fileno(f1), ftell(f1));
     fprintf(stderr,"\nread-PNM-error: bad magic bytes, expect 0x50 0x3[1-6]"
                    " but got 0x%02x 0x%02x", 255&c1, 255&c2);
-    if (f1) close(f1); f1=NULL; return(-1);
+    if (f1) fclose(f1); f1=NULL; return(-1);
   }
   nx=ny=nc=0; if (c2=='4' || c2=='1') nc=1;
   for(mod=0;((c2=='5' || c2=='2') && (mod&7)<6)
@@ -440,7 +441,7 @@ int writeppm(char *nam, pix *p){ /* P6 raw-ppm */
 #ifdef HAVE_POPEN
   /* be sure that nam contains hacker code like "dummy | rm -rf *" */
   if (!f1) {
-    strncpy(buf,"pnmtopng > ",12);
+    strncpy(buf,"pnmtopng > ",12);  /* no spaces within filenames allowed! */
     strncpy(buf+11,nam,111); buf[123]=0;
     strncpy(buf+strlen(buf),".png",5); 
     f1 = popen(buf, "w"); if(f1) f1t=1; else E0("popen pnmtopng");
